/* wlandump.c: Entry point to frame 'snif' and dump program
* ----------------------------------------------------------------
*
* The contents of this file are subject to the Mozilla Public
* License Version 1.0 (the "License"); you may not use this file
* except in compliance with the License. You may obtain a copy of
* the License at http://www.mozilla.org/MPL/
*
* Software distributed under the License is distributed on an "AS
* IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
* implied. See the License for the specific language governing
* rights and limitations under the License.
*
* The initial developer of the original code is Jo-Ellen F. Mathews
* <joellen@absoval.com>.  Portions were created by Jo-Ellen F.
* Mathews and Mark S. Mathews and are Copyright (C) 1998
* AbsoluteValue Software, Inc.  All Rights Reserved.
*
----------------------------------------------------------------*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* Ugly hack for LinuxPPC R4, don't have time to figure it out right now */
#if defined(_ARCH_PPC)
#undef __GLIBC__
#endif

#include <sys/socket.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/param.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>

#include <asm-i386/types.h>
#include <linux/netlink.h>

#include <wlan/version.h>
#include <wlan/wlan_compat.h>
#include <wlan/p80211types.h>
#include <wlan/p80211hdr.h>
#include <wlan/p80211msg.h>
#include <wlan/p80211ioctl.h>
#include <wlan/p80211metastruct.h>
#include <wlan/p80211metadef.h>
#include "wlandump.h"


#define APPNAME		"wlandump"


int	read_frames_from_file( char *filename );
void	usage(void);
void 	handlesig(int signo);
void	sniff_interface(void);
void	strtoUINT40( UINT8 *a, char *s );
int	sighdlr_init(void);
int	netlink_init(void);
int	ioctl_init(void);
int	sniff_enable(char *ifname, int enable);

/*----------------------------------------------------------------
*	Global Variables
----------------------------------------------------------------*/
char			appname[APPNAME_MAX + 1];
char			*fullname;
char			ifname[16];
int			fd_ioctl = 0;	/* Used to send request messages */
int			fd_sniff = 0;	/* Used to rcv the sniffed frames */

/* option flags */
int	opt_nobeacon = 0;	/* -b => don't show beacons */
int	opt_showrx = 1;		/* -r => don't show rx'd frames (on by default) */
int	opt_showtx = 0;		/* -t => show tx'd frames */
int	opt_noshowctl = 0;	/* -c => don't show ctl frames */
int	opt_noshowmgmt = 0;	/* -m => don't show mgmt frames */
int	opt_noshowdata = 0;	/* -d => don't show data frames */
int	opt_showdatalen = 2312;	/* -s <len> => show, at most, this many data bytes */
int	opt_showraw = 0;	/* -x => show the whole frame as raw bytes */
int	opt_nodecrypt = 0;	/* -w => don't decrypt data frame contents */
int	opt_nodefrag = 0;	/* -u => don't unfragment */
int	opt_showfrags = 0;	/* -f => show fragments as well as defragged frm */
int opt_wepkeyset = 0;		/* -k <keyindex(0-3)>-xx:xx:xx:xx:xx  */
UINT8	wep_key[WLAN_WEP_NKEYS][WLAN_WEP_KEYLEN];

char				opts[] = "brtcmdxwufs:k:";

/*----------------------------------------------------------------
*	Main
----------------------------------------------------------------*/
int main(int argc, char *argv[])
{
	int	result = 0;
	int	optch;

	strcpy( appname, APPNAME );
	fullname = argv[0];

	/* if no args, print the usage msg */
	if ( argc < 2 ) {
		usage();
		return -1;
	}

	/* collect the args */
	while ( ((optch = getopt(argc, argv, opts)) != EOF) && (result == 0) ) {
		switch (optch)
		{
		case 'b':
			opt_nobeacon = 1;
			break;
		case 'r':
			opt_showrx = 0;
			break;
		case 't':
			opt_showtx = 0;
			break;
		case 'c':
			opt_noshowctl = 1;
			break;
		case 'm':
			opt_noshowmgmt = 1;
			break;
		case 'd':
			opt_noshowdata = 1;
			break;
		case 's':
			opt_showdatalen = atoi(optarg);
			break;
		case 'x':
			opt_showraw = 1;
			break;
		case 'w':
			opt_nodecrypt = 1;
			break;
		case 'u':
			opt_nodefrag = 1;
			break;
		case 'f':
			opt_showfrags = 1;
			break;
		case 'k':
			opt_wepkeyset = 1;
			strtoUINT40( wep_key[optarg[0] - '0'], optarg + 2);
			break;
		case 'v':
			printf("%s utility version %s\n", appname, WLAN_RELEASE);
			return 0;
			break;
		default:
			usage();
			return -1;
			break;
		}
	}

	/* if we made it this far, the options must be pretty much OK */
	if ( optind >= argc ) {
		/* we're missing the interface argument */
		fprintf(stderr, "%s: missing argument - interface\n", fullname);
		usage();
		return -1;
	} else {
		/* Make the crc table for decrypt to use */
		wlandump_crc32_mktable();

		/* save the interface name */
		strcpy( ifname, argv[optind] );

		/* Set up the signal handler */
		if ( sighdlr_init() != 0 ) {
			fprintf(stderr,"Failed to set up signal handler, abort.\n");
			return -1;
		}

		/* Open the netlink socket, sets fd_sniff (close occurs in sighdlr) */
		if ( netlink_init() != 0 ) {
			fprintf(stderr,"Failed to open the netlink socket, abort.\n");
			return -1;
		}

		/* Open the ioctl socket, sets fd_ioctl */
		if ( ioctl_init() != 0 ) {
			fprintf(stderr,"Failed to open the ioctl socket, abort.\n");
			return -1;
		}

		/* Enable the sniffing */
		if ( sniff_enable(ifname, 1) != 0 ) {
			fprintf(stderr,"Failed to enable sniffing, abort.\n");
			return -1;
		}

		/* Read from the netlink socket (forever) */
		sniff_interface();
	}

	/* if we reach here, something went wrong in sniff_interface, return error */
	return -1;
}


/*----------------------------------------------------------------
* sniff_enable
*
* Sends the sniff enable request message to the interface of
* of interest.
*
* Arguments:
*	ifname	- interface name we wish to sniff
*	enable	- !0=enable sniffing, 0=disable sniffing
* Returns: 
*	0	success 
*	!0	failure
----------------------------------------------------------------*/
int sniff_enable(char *ifname, int enable)
{
	int				result = 0;
	p80211ioctl_req_t		req;
	p80211msg_lnxreq_wlansniff_t	msg;

	/* Set up the message */
	msg.msgcode = DIDmsg_lnxreq_wlansniff;
	msg.msglen = sizeof(msg);
	strcpy( msg.devname, ifname);
	msg.enable.did = DIDmsg_lnxreq_wlansniff_enable;
	msg.enable.status = P80211ENUM_msgitem_status_data_ok;
	msg.enable.len = 4;
	msg.enable.data = enable ? 
		P80211ENUM_truth_true : P80211ENUM_truth_false;
	msg.resultcode.did = DIDmsg_lnxreq_wlansniff_resultcode;
	msg.resultcode.len = 4;
	msg.resultcode.status = P80211ENUM_msgitem_status_no_value;
	msg.resultcode.data = 0;

	/* Set up the req struct */
	req.magic = P80211_IOCTL_MAGIC;
	req.len = sizeof(msg);
	req.data = &msg;
	strcpy( req.name, ifname);
	req.result = 0;

	result = ioctl( fd_ioctl, P80211_IFREQ, &req);

	if ( result == -1 ) {
	     	fprintf(stderr,"sniff_enable: ioctl failed\n");
		perror(appname);
		return -1;
	}

	/* Check the resultcode field of the message */
	if ( msg.resultcode.status != P80211ENUM_msgitem_status_data_ok ||
	     msg.resultcode.data != P80211ENUM_resultcode_success ) {
	     fprintf(stderr,"msg.resultcode failure\n");
	     return -1;
	}

	return 0;
}


/*----------------------------------------------------------------
* ioctl_init
*
* Opens the socket we'll use for issuing ioctls to the interface
* we wish to sniff.  Socket fd is stored in global 'fd_ioctl'
*
* Arguments:
*	none
* Returns: 
*	0	success 
*	!0	failure
----------------------------------------------------------------*/
int ioctl_init(void)
{
	fd_ioctl = socket(AF_INET, SOCK_STREAM, 0);
	if ( fd_ioctl == -1 )
	{
	     	fprintf(stderr,"ioctl_init: socket failed\n");
		perror(appname);
		return -1;
	}
	return 0;
}


/*----------------------------------------------------------------
* netlink_init
*
* Opens the netlink socket the will be used to read the sniffed
* frames.  The resulting file descriptor is stored in the global
* variable 'snifffd'
* the named device is, in fact, a wireless interface.
*
* Arguments:
*	none
* Returns: 
*	0	success 
*	!0	failure
----------------------------------------------------------------*/
int netlink_init(void)
{
	struct sockaddr_nl	nlskaddr;

	/* open the netlink socket */
	if ( (fd_sniff = socket( PF_NETLINK, SOCK_RAW, P80211_NL_SOCK_IND )) == -1 ) {
	     	fprintf(stderr,"netlink_init: socket failed\n");
		perror(fullname);
		return -1;
	} 
	
	memset ( &nlskaddr, 0 , sizeof( nlskaddr ));
	nlskaddr.nl_family = (sa_family_t)PF_NETLINK;
	nlskaddr.nl_pid = (__u32)getpid();
	nlskaddr.nl_groups = P80211_NL_MCAST_GRP_SNIFF;

	if ((bind( fd_sniff, (struct sockaddr*)&nlskaddr, sizeof(nlskaddr))) == -1 ) {
	     	fprintf(stderr,"netlink_init: bind failed\n");
		perror(fullname);
		return -1;
	}
	return 0;
}


/*----------------------------------------------------------------
* sniff_interface
*
* Loops forever reading and printing frames.
*
* Arguments:
*	none
*
* Returns: 
*	should never return.  Program terminates via signal
*	handler.
----------------------------------------------------------------*/
void sniff_interface(void)
{
	p80211msg_lnxind_wlansniffrm_t	*msg;

	UINT8		snifbuf[WLAN_A4FR_MAXLEN + sizeof(p80211msg_lnxind_wlansniffrm_t)];
	UINT8		*frame;
	int		recvlen;

	/* loop forever */
	while(1) {
		recvlen = recv( fd_sniff, snifbuf, sizeof(snifbuf), 0 );


		if ( recvlen > -1 ) {
			msg = (p80211msg_lnxind_wlansniffrm_t*)snifbuf;
			frame = snifbuf + sizeof(p80211msg_lnxind_wlansniffrm_t);
			if ( recvlen != (msg->msglen + msg->frmlen.data) ) {
	     			fprintf(stderr,
					"sniff_interface: recvlen != msglen + frmlen\n"
					"recvlen=%lu, msglen+frmlen=%lu",
					(unsigned long int)recvlen,
					(unsigned long int)(msg->msglen + msg->frmlen.data));

			}
			/*
			print_80211_frame( msg, frame );
			*/
		} else {
	     		fprintf(stderr,"sniff_interface: recv failed\n");
			perror(fullname);
		}
	}
	return;
}


/*----------------------------------------------------------------
* sighdlr_init
*
* Assigns the one handler to all catchable signals
*
* Arguments:
*	none
*
* Returns:
*	0	success
----------------------------------------------------------------*/
int sighdlr_init(void)
{
	int	i;

	/* set up the signal handler */
	for ( i = 1; i < _NSIG; i++) {
		signal( i, handlesig);
	}
	return 0;
}


/*----------------------------------------------------------------
* handlesig
*
* Cleans up after the sniffer.
*
* Arguments:
*	signo	- signal number
*
* Returns:
*	nothing
----------------------------------------------------------------*/
void handlesig(int signo)
{
	sniff_enable(ifname, 0);
	close(fd_ioctl);
	close(fd_sniff);
	exit(0);
}


/*----------------------------------------------------------------
* strtoUINT40
*
* This function converts a character string that represents an
* a 40 bit quantity.
*
* Arguments:
*	a	40 bit buffer
*	s	character string (make sure it's big enough!)
*
* Returns: 
*	nothing
----------------------------------------------------------------*/
void strtoUINT40( UINT8 *a, char *s )
{
	char	*p;
	int	i;
	UINT	val;

	for ( i = 0; i < 4; i++) {
		p = strchr( s, ':');
		if ( p == NULL ) {
			fprintf(stderr, "%s: UINT48 format error!\n", appname);
			exit(1);
		} else {
			*p = '\0';
			sscanf( s, "%x", &val);
			a[i] = (UINT8)val;
			s = p+1;
		}
	}
	sscanf( s, "%x", &val);
	a[i] = (UINT8)val;
	return;
}



/*----------------------------------------------------------------
* usage
*
* This function displays the proper syntax of the frdumper utility.
*
* Arguments:
*	none
*
* Returns:
*	nothing
----------------------------------------------------------------*/
void usage(void)
{
	printf("\n%s : 802.11 frame dump utility\n", appname);
	printf("    usage: %s [option ...] interface\n\n", appname);
	printf("          where valid options are:\n");
	printf("          -b        don't display beacon frames\n");
	printf("          -r        don't show received frames (shown by default)\n");
	printf("          -t        show transmitted frames\n");
	printf("          -c        don't show ctl frames\n");
	printf("          -m        don't show mgmt frames\n");
	printf("          -d        don't show data frames\n");
	printf("          -s <len>  show, at most, this many data field bytes\n");
	printf("          -x        show the whole frame as raw bytes\n");
	printf("          -w        don't decrypt data frame contents\n");
	printf("          -u        don't unfragment\n");
	printf("          -f        show fragments as well as defragged frame\n");
	printf("          -k <<keyindex(0-3)>-xx:xx:xx:xx:xx>\n");
	printf("                    set wep key[keyindex], don't foget to set all 4 keys\n");
	printf("          -v  display the program version and exit.\n\n");
	printf("  Note: this program doesn't do a whole lot of cmdline argument validation\n");
	printf("        so be careful when setting them.\n\n");
}


